@@ -4,8 +4,88 @@ import pandas as pd
 import numpy as np
 from . import stats
 from math import ceil
+  
+def drawdown_details(drawdown):
+  """
+  Calculates drawdown details, including start/end/valley dates,
+  duration, max drawdown and max dd for 99% of the dd period
+  for every drawdown period
+  """
+  def _drawdown_details(drawdown):
+    # mark no drawdown
+    no_dd = drawdown == 0
 
-# """lorem"""
+    # extract dd start dates
+    starts = ~no_dd & no_dd.shift(1)
+    starts = list(starts[starts].index)
+
+    # extract end dates
+    ends = no_dd & (~no_dd).shift(1)
+    ends = list(ends[ends].index)
+
+    # no drawdown :)
+    if not starts:
+        return pd.DataFrame(
+            index=[], columns=('start', 'valley', 'end', 'days',
+                                'max drawdown', '99% max drawdown'))
+
+    # drawdown series begins in a drawdown
+    if ends and starts[0] > ends[0]:
+        starts.insert(0, drawdown.index[0])
+
+    # series ends in a drawdown fill with last date
+    if not ends or starts[-1] > ends[-1]:
+        ends.append(drawdown.index[-1])
+
+    # build dataframe from results
+    data = []
+    for i, _ in enumerate(starts):
+        dd = drawdown[starts[i]:ends[i]]
+        clean_dd = -utils.remove_outliers(-dd, .99)
+        data.append((starts[i], dd.idxmin(), ends[i],
+                      (ends[i] - starts[i]).days,
+                      dd.min() * 100, clean_dd.min() * 100))
+
+    df = pd.DataFrame(data=data,
+                        columns=('start', 'valley', 'end', 'days',
+                                'max drawdown',
+                                '99% max drawdown'))
+    df['days'] = df['days'].astype(int)
+    df['max drawdown'] = df['max drawdown'].astype(float)
+    df['99% max drawdown'] = df['99% max drawdown'].astype(float)
+
+    df['start'] = df['start'].dt.strftime('%Y-%m-%d')
+    df['end'] = df['end'].dt.strftime('%Y-%m-%d')
+    df['valley'] = df['valley'].dt.strftime('%Y-%m-%d')
+
+    return df
+
+  if isinstance(drawdown, pd.DataFrame):
+    _dfs = {}
+    for col in drawdown.columns:
+      _dfs[col] = _drawdown_details(drawdown[col])
+    return pd.concat(_dfs, axis=1)
+  return _drawdown_details(drawdown)
+
+def to_pct (input, dec=1):
+  if dec == 2:
+    dec = '{:,.2f}%'
+  elif dec == 3:
+    dec = '{:,.3f}%'
+  else:
+    dec = '{:,.1f}%'
+  return (input*100).map(dec.format)
+
+def to_num (input, dec=1):
+  if dec == 2:
+    dec = '{:,.2f}'
+  elif dec == 3:
+    dec = '{:,.3f}'
+  elif dec == 0:
+    dec = '{:0.0f}'
+  else:
+    dec = '{:,.1f}'
+  return input.map(dec.format)
 
 # date manipulation
 # the df.index should be datetime 
@@ -137,12 +217,16 @@ def clean(data):
 
 def to_quotes(returns, base=1e5):
   """converts returns series to quote prices"""
-  return (1+ stats.rolling_returns(returns))* base
+  return (1+ to_rolling_returns(returns))* base
 
 def to_returns(prices):
   """simple arithmetic returns from a price series"""
   return prices.pct_change()
 
+def to_rolling_returns(returns):
+  """calculates rolling compounded returns"""
+  return returns.add(1).cumprod() -1
+  
 def to_log_returns(returns, base='e'):
   """logarithm returns from a price series and base"""
   if base == '2':
